@@ -48,6 +48,7 @@ def run_careless(parser):
         model.surrogate_posterior.trainable = False
 
     validation_frequency = parser.validation_frequency
+    progress = not parser.disable_progress_bar
 
     history = model.train_model(
         tuple(map(tf.convert_to_tensor, train)),
@@ -55,6 +56,7 @@ def run_careless(parser):
         message="Training",
         validation_data=test,
         validation_frequency=validation_frequency,
+        progress=progress,
     )
 
     for i,ds in enumerate(dm.get_results(model.surrogate_posterior, inputs=train)):
@@ -73,21 +75,16 @@ def run_careless(parser):
     predictions_data = None
     if test is not None:
         for file_id, (ds_train, ds_test) in enumerate(zip(
-                dm.get_predictions(model, train),
-                dm.get_predictions(model, test),
+                dm.get_predictions(model, train, test_value=0),
+                dm.get_predictions(model, test, test_value=1),
                 )):
-            ds_train['test'] = rs.DataSeries(0, index=ds_train.index, dtype='I')
-            ds_test['test']  = rs.DataSeries(1, index=ds_test.index, dtype='I')
-
             filename = parser.output_base + f'_predictions_{file_id}.mtz'
             rs.concat((
                 ds_train,
                 ds_test,
             )).write_mtz(filename)
     else:
-        for file_id, ds_train in enumerate(dm.get_predictions(model, train)):
-            ds_train['test'] = rs.DataSeries(0, index=ds_train.index, dtype='I')
-
+        for file_id, ds_train in enumerate(dm.get_predictions(model, train, test_value=0)):
             filename = parser.output_base + f'_predictions_{file_id}.mtz'
             ds_train.write_mtz(filename)
 
@@ -102,6 +99,7 @@ def run_careless(parser):
                     tuple(map(tf.convert_to_tensor, half)), 
                     parser.iterations,
                     message=f"Merging repeat {repeat+1} half {half_id+1}",
+                    progress=progress,
                 )
 
                 for file_id,ds in enumerate(dm.get_results(model.surrogate_posterior, inputs=half)):
