@@ -211,7 +211,7 @@ class MonoFormatter(DataFormatter):
 
         return cls(
             parser.intensity_key,
-            None, #<-- uncertainty key has to match {SIG,Sig}intensity_key
+            parser.uncertainty_key,
             parser.image_key,
             parser.metadata_keys.split(','),
             parser.separate_files,
@@ -288,11 +288,9 @@ class MonoFormatter(DataFormatter):
                 for k in ds:
                     if k == prefix + intensity_key:
                         uncertainty_key = k
-        # TODO: this is a temporary fix for an older version of the crystfel parser
-        # please remove this upon the next rs release (2021-09-10)
         if uncertainty_key is None:
-            if 'sigmaI' in ds:
-                uncertainty_key = 'sigmaI'
+            uncertainty_key = get_first_key_of_dtype(ds, 'Q')
+
 
         if uncertainty_key is None:
             raise ValueError(
@@ -337,6 +335,7 @@ class MonoFormatter(DataFormatter):
             encoded  = positional_encoding(to_encode, self.ecoding_bit_depth)
             metadata = np.concatenate((metadata, encoded), axis=1)
 
+        file_id = data['file_id'].to_numpy('int64')
         refl_id = rac.to_refl_id(
             data['asu_id'].to_numpy('int64')[:,None],
             data.get_hkls(),
@@ -347,6 +346,7 @@ class MonoFormatter(DataFormatter):
 
         inputs = {
             'refl_id'   : refl_id[:,None],
+            'file_id'   : file_id[:,None],
             'image_id'  : data['image_id'].to_numpy('int64')[:,None],
             'metadata'  : metadata,
             'intensities'   : iobs,
@@ -443,7 +443,7 @@ class LaueFormatter(DataFormatter):
         return cls(
             parser.wavelength_key,
             parser.intensity_key,
-            None, #<-- uncertainty key has to match {SIG,Sig}intensity_key
+            parser.uncertainty_key,
             parser.image_key,
             parser.metadata_keys.split(','),
             parser.separate_files,
@@ -536,11 +536,8 @@ class LaueFormatter(DataFormatter):
                 for k in ds:
                     if k == prefix + intensity_key:
                         uncertainty_key = k
-        # TODO: this is a temporary fix for an older version of the crystfel parser
-        # please remove this upon the next rs release (2021-09-10)
         if uncertainty_key is None:
-            if 'sigmaI' in ds:
-                uncertainty_key = 'sigmaI'
+            uncertainty_key = get_first_key_of_dtype(ds, 'Q')
 
         if uncertainty_key is None:
             raise ValueError(
@@ -589,6 +586,7 @@ class LaueFormatter(DataFormatter):
             encoded  = positional_encoding(to_encode, self.ecoding_bit_depth)
             metadata = np.concatenate((metadata, encoded), axis=1)
 
+        file_id = data['file_id'].to_numpy('int64')
         refl_id = rac.to_refl_id(
             data['asu_id'].to_numpy('int64')[:,None],
             data.get_hkls(),
@@ -601,6 +599,7 @@ class LaueFormatter(DataFormatter):
 
         inputs = {
             'refl_id'   : refl_id[:,None],
+            'file_id'   : file_id[:,None],
             'image_id'  : data['image_id'].to_numpy('int64')[:,None],
             'metadata'  : metadata,
             'intensities'   : iobs,
