@@ -7,7 +7,7 @@ import numpy as np
 
 
 class NormalLayer(tf.keras.layers.Layer):
-    def __init__(self, scale_bijector=None, epsilon=1e-32, **kwargs): 
+    def __init__(self, scale_bijector=None, epsilon=1e-7, **kwargs): 
         super().__init__(**kwargs)
         self.epsilon = epsilon
         if scale_bijector is None:
@@ -28,7 +28,7 @@ class MetadataScaler(Scaler):
     Neural network based scaler with simple dense layers.
     This neural network outputs a normal distribution.
     """
-    def __init__(self, n_layers, width, leakiness=0.01):
+    def __init__(self, n_layers, width, leakiness=0.01, epsilon=1e-7):
         """
         Parameters
         ----------
@@ -73,7 +73,7 @@ class MetadataScaler(Scaler):
 
         #The final layer converts the output to a Normal distribution
         #tfp_layers.append(tfp.layers.IndependentNormal())
-        tfp_layers.append(NormalLayer())
+        tfp_layers.append(NormalLayer(epsilon=epsilon))
 
         self.network = tf.keras.Sequential(mlp_layers)
         self.distribution = tf.keras.Sequential(tfp_layers)
