@@ -1,12 +1,36 @@
 import pytest
-from os import listdir
-from os.path import dirname, abspath, join
+from os import listdir,mkdir
+from os.path import dirname, abspath, join, exists
 import numpy as np
 import pandas as pd
 import re
 import reciprocalspaceship as rs
 import gemmi
 
+def pytest_sessionstart(session):
+    rundir = "data/"
+    rundir = abspath(join(dirname(__file__), rundir))
+
+    command = """
+    careless poly 
+        --disable-progress-bar 
+        --iterations=10 
+        --merge-half-datasets 
+        --half-dataset-repeats=3 
+        --test-fraction=0.1 
+        --disable-gpu 
+        --anomalous 
+        --wavelength-key=Wavelength
+        dHKL,Hobs,Kobs,Lobs,Wavelength
+        pyp_off.mtz 
+        pyp_2ms.mtz 
+        output/pyp
+    """
+    if not exists(f"{rundir}/output"):
+        mkdir(f"{rundir}/output")
+        from subprocess import call
+        call(command.split(), cwd=rundir)
+
 @pytest.fixture
 def cell_and_spacegroups():
     data = [
@@ -25,6 +49,12 @@ def xval_mtz():
     filename = abspath(join(dirname(__file__), datapath))
     return filename
 
+@pytest.fixture
+def history_csv():
+    datapath = "data/output/pyp_history.csv"
+    filename = abspath(join(dirname(__file__), datapath))
+    return filename
+
 @pytest.fixture
 def predictions_mtz():
     datapath = "data/output/pyp_predictions_0.mtz"
@@ -112,9 +142,11 @@ class LaueTestData():
         self.data = ds
         self.reciprocal_asu = rasu
         self.reciprocal_asu_collection = rasu_collection
+        file_id = np.zeros_like(refl_id)
         self.inputs = [
             refl_id,
             image_id,
+            file_id,
             metadata,
             intensities,
             uncertainties,
@@ -167,9 +199,11 @@ class MonoTestData():
         self.data = ds
         self.reciprocal_asu = rasu
         self.reciprocal_asu_collection = rasu_collection
+        file_id = np.zeros_like(refl_id)
         self.inputs = [
             refl_id,
             image_id,
+            file_id,
             metadata,
             intensities,
             uncertainties,
